var ExtractionReg = function()
{
	var exreg = {};
	var debug = 0;
	
	var latestRna;
	var latestLysate;
	
	exreg.REAGENTS = [
		'rltPlusBuffer', 'betaMercaptoetanol', 'reagentDx', 'qiaShredderColumn',
		'allPrepMiniKit', 'etoh70', 'bufferRw1', 'bufferRpe', 'bufferAw1', 
		'bufferAw2', 'bufferEb', 'rnaseFreeWater', 'allPrepDnaColumn', 'rneasyColumn' 
		];

	
	var DEFAULT_LYSATE_VOLUME = 700;
	var DEFAULT_LYSATE_USED = 350;
	var DEFAULT_DNA_VOLUME = 95;
	var DEFAULT_RNA_VOLUME = 51;
	var DEFAULT_FT_VOLUME = 550;
	var MINIMAL_PRE_NORMALIZE_QUANTITY = 0; // µg, =0 since we no longer handle NotEnoughQuantity in this wizard

	var lysatesIsValid = false;
	var qiacubeDateIsAfterLysisDate = true;
	var qiacubeRunNoIsValid = false;
	var qiacubePositionsAreValid = false;
	var nanoDropFileIsValid = false;
	var createdRna = [];
	
	// Page initialization
	exreg.initPage = function()
	{
		// Step 1
		Events.addEventHandler('step-1', 'wizard-validate', exreg.validateStep1);
		Events.addEventHandler('lysates', 'change', exreg.lysatesOnChange);
		
		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', exreg.initializeStep2);
		Events.addEventHandler('step-2', 'wizard-validate', exreg.validateStep2);
		
		Events.addEventHandler('lysisDate', 'change', Wizard.validateDate);
		Events.addEventHandler('qiacubeDate', 'change', Wizard.validateDate);
		Events.addEventHandler('lysisDate', 'change', exreg.qiacubeDateOnChange);
		Events.addEventHandler('qiacubeDate', 'change', exreg.qiacubeDateOnChange);
		Events.addEventHandler('qiacubeRunNo', 'change', exreg.qiacubeRunNoOnChange);
		
		Buttons.addClickHandler('btnUseLatestLysateReagents', exreg.useLatestReagents);
		Buttons.addClickHandler('btnUseLatestRnaReagents', exreg.useLatestReagents);
		
		// Step 3
		Events.addEventHandler('step-3', 'wizard-initialize', exreg.initializeStep3);
		Events.addEventHandler('step-3', 'wizard-validate', exreg.validateStep3);
	
		Buttons.addClickHandler('downloadSampleIdFile', exreg.downloadSampleIdFile);
		Buttons.addClickHandler('btnFakeNanoFropFile', exreg.generateFakeNanoDropFile);
		Events.addEventHandler('nanoDropFile', 'change', exreg.nanoDropFileOnChange);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
		Buttons.addClickHandler('rnaQcWizard', exreg.goRnaQc);
		
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', exreg.submit);

		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetUnprocessedLysates';		
		Wizard.showLoadingAnimation('Loading Lysate extracts...');
		Wizard.asyncJsonRequest(url, exreg.initializeStep1);
	}
	
	exreg.initializeStep1 = function(response)
	{
		var frm = document.forms['reggie'];
		var lysates = response.lysates;
		
		var lysateList = frm.lysates;
		if (lysates.length > 0)
		{
			for (var i=0; i < lysates.length; i++)
			{
				var lysate = lysates[i];
				var name = (i+1) + ': ' + Strings.encodeTags(lysate.name);
				if (lysate.bioWell)
				{
					name += ' -- ' + Strings.encodeTags(lysate.bioWell.bioPlate.name + ' (' + lysate.bioWell.location+')');
				}
				var selected = i < 12;
				var option = new Option(name, lysate.id, selected, selected);
				option.lysate = lysate;
				lysateList.options[lysateList.length] = option;
			}
			exreg.lysatesOnChange();
		}
		else
		{
			Wizard.setFatalError('No Lysate available for processing.');
			return;
		}

		Doc.show('step-1');
		Doc.show('gonext');
	}

	
	exreg.lysatesOnChange = function()
	{
		lysatesIsValid = false;
		Wizard.setInputStatus('lysates');
		
		var frm = document.forms['reggie'];
		var lysateList = frm.lysates;
		
		var numSelected = 0;
		for (var i = 0; i < lysateList.length; i++)
		{
			if (lysateList[i].selected) numSelected++;
		}
		
		if (numSelected != 12)
		{
			Wizard.setInputStatus('lysates', 'invalid', 'Must select 12 items.');
			return;
		}
		
		lysatesIsValid = true;
		Wizard.setInputStatus('lysates', 'valid');
	}

	
	exreg.validateStep1 = function(event)
	{
		if (!lysatesIsValid)
		{
			event.preventDefault();
		}
	}
	
	exreg.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		frm.lysates.disabled = true;
		
		Wizard.setCurrentStep(2);
		Doc.show('gocancel');
		Doc.show('gonext');
		
		Reggie.loadProtocols('SAMPLE_HANDLING_PROTOCOL', 'lysisProtocol');
		Doc.addClass('dnaProtocol', 'list-loading');
		frm.dnaProtocol[0] = new Option('loading...', '');
		Doc.addClass('rnaProtocol', 'list-loading');
		frm.rnaProtocol[0] = new Option('loading...', '');
		Doc.addClass('ftProtocol', 'list-loading');
		frm.ftProtocol[0] = new Option('loading...', '');
		var url = '../Protocol.servlet?ID='+App.getSessionId();
		url += '&cmd=GetProtocols&subtype=EXTRACTION_PROTOCOL';
		Wizard.asyncJsonRequest(url, exreg.protocolsLoaded);

		frm.lysisDate.focus();
		Events.sendChangeEvent('lysisDate');
		Events.sendChangeEvent('qiacubeDate');
		
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetLatestReagentLotNumbers';		
		Wizard.showLoadingAnimation('Loading recent reagent lot numbers...');
		Wizard.asyncJsonRequest(url, exreg.reagentsLoaded);
	}
	
	exreg.protocolsLoaded = function(response)
	{
		var frm = document.forms['reggie'];
		var protocols = response.protocols;

		Doc.removeClass('dnaProtocol', 'list-loading');
		Doc.removeClass('rnaProtocol', 'list-loading');
		Doc.removeClass('ftProtocol', 'list-loading');
		frm.dnaProtocol.length = 0;
		frm.rnaProtocol.length = 0;
		frm.ftProtocol.length = 0;
		for (var i = 0; i < protocols.length; i++)
		{
			var name = protocols[i].name;
			var id = protocols[i].id;
			frm.dnaProtocol[frm.dnaProtocol.length] = new Option(name, id, false, name == 'AllprepCustomPartA');
			frm.rnaProtocol[frm.rnaProtocol.length] = new Option(name, id, false, name == 'AllprepMod1partC');
			frm.ftProtocol[frm.ftProtocol.length] = new Option(name, id, false, name == 'AllprepMod1partB');
		}
		
		frm.dnaProtocol[frm.dnaProtocol.length] = new Option('- none -', '');
		frm.rnaProtocol[frm.rnaProtocol.length] = new Option('- none -', '');
		frm.ftProtocol[frm.ftProtocol.length] = new Option('- none -', '');
	}
	
	exreg.reagentsLoaded = function(response)
	{
		latestRna = response.rna;
		latestLysate = response.lysate;
		
		if (!latestRna) return;
		
		var haveLatestLysateReagents = false;
		var haveLatestRnaReagents = false;
		for (var rgNo = 0; rgNo < exreg.REAGENTS.length; rgNo++)
		{
			var rg = exreg.REAGENTS[rgNo];
			if (latestLysate[rg])
			{
				haveLatestLysateReagents = true;
			}
			if (latestRna[rg])
			{
				haveLatestRnaReagents = true;
			}
		}
		
		if (haveLatestLysateReagents)
		{
			Doc.show('useLatestLysateReagents');
			Doc.element('latestLysisDate').innerHTML = '(Lysis date ' + Reggie.reformatDate(latestLysate.lysisDate) + ')';
		}
		else
		{
			latestLysate = null;
		}
		
		if (haveLatestRnaReagents)
		{
			Doc.show('useLatestRnaReagents');
			Doc.element('latestQiacubeDate').innerHTML = '(QiaCube run ' + Reggie.reformatDate(latestRna.QiaCubeDate) + ')';
		}
		else
		{
			latestRna = null;
		}
	}
	
	exreg.useLatestReagents = function(event)
	{
		var which = Data.get(event.currentTarget, 'which');
		
		var latestItem = which == 'rna' ? latestRna : latestLysate;
		if (!latestItem) return;
		
		var frm = document.forms['reggie'];
		for (var rgNo = 0; rgNo < exreg.REAGENTS.length; rgNo++)
		{
			var rg = exreg.REAGENTS[rgNo];
			if (latestItem[rg])
			{
				frm[rg].value = latestItem[rg];
			}
		}
	}
	
	exreg.qiacubeDateOnChange = function()
	{
		var frm = document.forms['reggie'];
		
		var lysisDateIsValid = Wizard.isValid('lysisDate');
		var qiacubeDateIsValid = Wizard.isValid('qiacubeDate');
		
		if (!lysisDateIsValid || !qiacubeDateIsValid) return;

		var lysisDate = Dates.parseString(frm.lysisDate.value, 'yyyyMMdd');
		var qiacubeDate = Dates.parseString(frm.qiacubeDate.value, 'yyyyMMdd');
		if (lysisDate > qiacubeDate)
		{
			Wizard.setInputStatus('qiacubeDate', 'invalid', 'Qiacube date is before Lysis date');
			qiacubeDateIsAfterLysisDate = false;
			return;
		}

		qiacubeDateIsAfterLysisDate = true;
		
		Wizard.setInputStatus('qiacubeDate', 'valid');
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=GetNextQiacubeRunNo&qiacubeDate='+encodeURIComponent(frm.qiacubeDate.value);
		Wizard.showLoadingAnimation('Loading Qiacube run number...');
		Wizard.asyncJsonRequest(url, exreg.gotNextRunNumber);
	}

	exreg.gotNextRunNumber = function(response)
	{
		var frm = document.forms['reggie'];
		frm.qiacubeRunNo.value = response.nextQiacubeRunNo;
		Events.sendChangeEvent('qiacubeRunNo');
	}

	exreg.qiacubeRunNoOnChange = function()
	{
		var frm = document.forms['reggie'];
		var runNumber = frm.qiacubeRunNo.value;
		qiacubeRunNoIsValid = false;

		if (runNumber == '')
		{
			Wizard.setInputStatus('qiacubeRunNo', 'invalid', 'Missing');
			return;
		}
		if (!(parseInt(runNumber, 10) >= 1))
		{
			Wizard.setInputStatus('qiacubeRunNo', 'invalid', 'Must be at least 1');
			return;		
		}
		
		Wizard.setInputStatus('qiacubeRunNo', 'valid');
		qiacubeRunNoIsValid = true;
	}

	
	exreg.validateStep2 = function(event)
	{
		var valid = true;
		valid &= Wizard.isValid('lysisDate');
		valid &= Wizard.isValid('qiacubeDate');
		valid &= qiacubeRunNoIsValid;
		
		if (!valid) event.preventDefault();
	}

	
	exreg.initializeStep3 = function()
	{
		var frm = document.forms['reggie'];
		
		var html = '<table id="detailsTable" class="step-form">';
		// First header row
		html += '<thead>';
		html += '<tr>';
		html += '<th>Specimen</th>';
		html += '<th class="dottedleft">Qiacube</th>';
		html += '<th colspan="3" class="dottedleft">Lysate</th>';
		html += '<th colspan="3" class="dottedleft">DNA</th>';
		html += '<th colspan="5" class="dottedleft">RNA</th>';
		html += '<th class="dottedleft">FT</th>';
		html += '<th class="dottedleft comment">Comment</th>';
		html += '<th></th>';
		html += '</tr>';
		// Second header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft">position</th>';
		html += '<td class="dottedleft">Storage</td><td>Total</td><td>Used</td>';
		html += '<td class="dottedleft">Pos</td><td>Vol.</td><td>Conc.</td>';
		html += '<td class="dottedleft">Pos</td><td>Vol.</td><td>Conc.</td><td>Quantity</td><td><span id="toggle-normalize-1" class="link" title="Toggle all">Pre-</span></td>';
		html += '<td class="dottedleft">Vol.</td>';
		html += '<td class="dottedleft comment"></td>';
		html += '<td></td>';
		html += '</tr>';
		// Third header row
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<td class="dottedleft">box</td><td>(µl)</td><td>(µl)</td>';
		html += '<td class="dottedleft"></td><td>(µl)</td><td>(ng/µl)</td>';
		html += '<td class="dottedleft"></td><td>(µl)</td><td>(ng/µl)</td><td>(µg)</td><td><span id="toggle-normalize-2" class="link" title="Toggle all">normalize</span></td>';
		html += '<td class="dottedleft">(µl)</td>';
		html += '<td class="dottedleft comment"></td>';
		html += '<td></td>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		// Set default values for DNA/RNA extraction
		var qiacubePos = 1;
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				var lysate = frm.lysates[i].lysate;
				lysate.originalQuantity = DEFAULT_LYSATE_VOLUME;
				lysate.processedQuantity = DEFAULT_LYSATE_USED;
				lysate.qiacubePosition = qiacubePos;
				lysate.originalQiacubePosition = qiacubePos;
				qiacubePos++;
				
				var rna = {};
				var dna = {};
				var ft = {};
				// DNA
				dna.name = lysate.name + '.d';
				dna.volume = DEFAULT_DNA_VOLUME;
				
				// RNA
				rna.name = lysate.name + '.r';
				rna.volume = DEFAULT_RNA_VOLUME;
				
				// FlowThrough
				ft.name = lysate.name + '.f';
				ft.volume = DEFAULT_FT_VOLUME;
				
				lysate.rna = rna;
				lysate.dna = dna;
				lysate.ft = ft;
				
				html += '<tr class="highlight">';
				html += '<th>' + Strings.encodeTags(lysate.name.replace(/\.l$/, '')) + '</th>';
				html += '<td id="qiacubePosition'+i+'" class="dottedleft"></td>';
				html += '<td class="dottedleft">'+Strings.encodeTags(lysate.bioWell.bioPlate.name + '[' + lysate.bioWell.location)+']</td>';
				html += '<td id="lysateTotal'+i+'"></td>';
				html += '<td id="lysateUsed'+i+'"></td>';
				html += '<td id="dnaPos'+i+'" class="dottedleft"></td>';
				html += '<td id="dnaVol'+i+'"></td>';
				html += '<td id="dnaConc'+i+'" class="bg-filled-50 italic"></td>';
				html += '<td id="rnaPos'+i+'" class="dottedleft"></td>';
				html += '<td id="rnaVol'+i+'"></td>';
				html += '<td id="rnaConc'+i+'" class="bg-filled-50 italic"></td>';
				html += '<td id="rnaQuantity'+i+'" class="bg-filled-50 italic"></td>';
				html += '<td><input type="checkbox" name="rnaPreNormalize'+i+'" id="rnaPreNormalize'+i+'" disabled>';
				html += '<span id="rnaPreNormalize'+i+'.status" class="status">&nbsp;</span></td>';
				html += '<td id="ftVol'+i+'" class="dottedleft"></td>';
				html += '<td id="comments'+i+'" class="dottedleft comment"></td>';
				html += '<td><div class="button basicbutton interactable edit-details" style="width: 5em;" data-index="'+i+'">Edit&hellip;</div></td>';
				html += '</tr>';
				
			}
		}
		html += '</tbody>';
		html += '</table>';
		Doc.element('detailsSection').innerHTML = html;
		
		var buttons = Doc.element('detailsSection').getElementsByClassName('edit-details');
		for (var i = 0; i < buttons.length; i++)
		{
			Buttons.addClickHandler(buttons[i], exreg.editDnaRnaDetails);
		}
		
		Events.addEventHandler('toggle-normalize-1', 'click', exreg.toggleNormalize);
		Events.addEventHandler('toggle-normalize-2', 'click', exreg.toggleNormalize);
		
		exreg.updateDetailsTable();
		
		Wizard.setCurrentStep(3);
		Doc.show('gocancel');
		Doc.show('goregister');
		
		Wizard.setInputStatus('nanoDropFile', 'invalid', 'No file selected');
		Wizard.keepSessionAlive();
		
		if (debug && location.protocol != 'https:') 
		{
			Wizard.showDebugTools();
		}
	}
	
	exreg.toggleNormalize = function()
	{
		var frm = document.forms['reggie'];
		var check = null;
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				if (!frm['rnaPreNormalize'+i].disabled)
				{
					if (check == null) check = !frm['rnaPreNormalize'+i].checked;
					frm['rnaPreNormalize'+i].checked = check;
				}
			}
		}
	}
	
	exreg.updateDetailsTable = function()
	{
		var frm = document.forms['reggie'];
		var hasQiacubePosition = [];
		var duplicatePosition = 0;
		
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				var lysate = frm.lysates[i].lysate;
				Doc.element('qiacubePosition'+i).innerHTML = lysate.qiacubePosition;
				Doc.addOrRemoveClass('qiacubePosition'+i, 'nondefault', lysate.qiacubePosition != lysate.originalQiacubePosition);
			
				if (hasQiacubePosition[lysate.qiacubePosition])
				{
					duplicatePosition = lysate.qiacubePosition;
				}
				else
				{
					hasQiacubePosition[lysate.qiacubePosition] = true;
				}
			
				// Calculate NanoDrop well from Qiacube position
				var row;
				var col;
				if (lysate.qiacubePosition > 8)
				{
					row = lysate.qiacubePosition - 9;
					col = 2;
				}
				else
				{
					row = lysate.qiacubePosition - 1;
					col = 1;
				}
				lysate.dna.well = Reggie.wellToAlpha(row) + col;
				lysate.rna.well = Reggie.wellToAlpha(row) + (col+2);
			
				Doc.element('lysateTotal'+i).innerHTML = lysate.originalQuantity;
				Doc.addOrRemoveClass('lysateTotal'+i, 'nondefault', lysate.originalQuantity != DEFAULT_LYSATE_VOLUME);

				Doc.element('lysateUsed'+i).innerHTML = lysate.processedQuantity;
				Doc.addOrRemoveClass('lysateUsed'+i, 'nondefault', lysate.processedQuantity != DEFAULT_LYSATE_USED);
			
				Doc.element('dnaPos'+i).innerHTML = lysate.dna.well;
				Doc.addOrRemoveClass('dnaPos'+i, 'nondefault', lysate.qiacubePosition != lysate.originalQiacubePosition);
				Doc.element('dnaVol'+i).innerHTML = lysate.dna.volume;
				Doc.addOrRemoveClass('dnaVol'+i, 'nondefault', lysate.dna.volume != DEFAULT_DNA_VOLUME);
				var dnaConc = Doc.element('dnaConc'+i);
				dnaConc.innerHTML = Reggie.formatNumber(lysate.dna.ndConc) || '-';
				dnaConc.title = '260/280=' + lysate.dna.nd260by280 + '\n260/230=' + lysate.dna.nd260by230;
			
				Doc.element('rnaPos'+i).innerHTML =  lysate.rna.well;
				Doc.addOrRemoveClass('rnaPos'+i, 'nondefault', lysate.qiacubePosition != lysate.originalQiacubePosition);
				Doc.element('rnaVol'+i).innerHTML = lysate.rna.volume;
				Doc.addOrRemoveClass('rnaVol'+i, 'nondefault', lysate.rna.volume != DEFAULT_RNA_VOLUME);
				var rnaConc = Doc.element('rnaConc'+i);
				rnaConc.innerHTML = Reggie.formatNumber(lysate.rna.ndConc) || '-';
				rnaConc.title = '260/280=' + lysate.rna.nd260by280 + '\n260/230=' + lysate.rna.nd260by230;
				
				lysate.rna.quantity = lysate.rna.ndConc * lysate.rna.volume / 1000;
				var enoughQuantity = lysate.rna.quantity >= MINIMAL_PRE_NORMALIZE_QUANTITY;
				lysate.rna.flag = enoughQuantity ? null : 'NotEnoughRemainingQuantity';
				
				frm['rnaPreNormalize'+i].disabled = !enoughQuantity;
				Doc.element('rnaQuantity'+i).innerHTML = Reggie.formatNumber(lysate.rna.quantity || null, null, 2) || '-';
				frm['rnaPreNormalize'+i].checked = enoughQuantity;
				var warning = lysate.rna.quantity && !enoughQuantity;
				Wizard.setInputStatus('rnaPreNormalize'+i, warning ? 'flag' : '', warning ? 'Not enough quantity' : '');
				
				Doc.element('ftVol'+i).innerHTML = lysate.ft.volume;
				Doc.addOrRemoveClass('ftVol'+i, 'nondefault', lysate.ft.volume != DEFAULT_FT_VOLUME);
			
				var cmnt = Doc.element('comments'+i);
				var tooltip = [];
				var comment = [];
				if (lysate.comment || lysate.dna.comment || lysate.rna.comment || lysate.ft.comment)
				{
					comment[0] = 'Yes';
					if (lysate.comment) tooltip[tooltip.length] = 'Lysate: ' + lysate.comment;
					if (lysate.dna.comment) tooltip[tooltip.length] = 'DNA: ' + lysate.dna.comment;
					if (lysate.rna.comment) tooltip[tooltip.length] = 'RNA: ' + lysate.rna.comment;
					if (lysate.ft.comment) tooltip[tooltip.length] = 'FlowThrough: ' + lysate.ft.comment;
				}
				
				var modifiedReagents = false;
				for (var rgNo = 0; rgNo < exreg.REAGENTS.length; rgNo++)
				{
					var rg = exreg.REAGENTS[rgNo];
					if (lysate[rg]) 
					{
						tooltip[tooltip.length] = Doc.element(rg+'.title').innerHTML + ': ' + Strings.encodeTags(lysate[rg]);
						modifiedReagents = true;
					}
				}
				if (modifiedReagents)
				{
					comment[comment.length] = 'Reagents';
				}
				
				if (tooltip.length > 0)
				{
					cmnt.innerHTML = comment.join(', ');
					Doc.addClass(cmnt, 'nondefault');
					cmnt.title = tooltip.join('\n');
				}
				else
				{
					cmnt.innerHTML = '';
					Doc.removeClass(cmnt, 'nondefault');
					cmnt.title = null;
				}
			}
		}
		
		if (duplicatePosition)
		{
			Wizard.setInputStatus('downloadSampleIdFile', 'invalid', 'Duplicate Qiacube position: ' + duplicatePosition);
			qiacubePositionsAreValid = false;
		}
		else
		{
			Wizard.setInputStatus('downloadSampleIdFile');
			qiacubePositionsAreValid = true;
		}
	}

	exreg.editDnaRnaDetails = function(event)
	{
		var index = Data.get(event.currentTarget, 'index');
		var frm = document.forms['reggie'];
		var url = 'edit_dnarna_details.jsp?ID='+App.getSessionId();
		url += '&index='+index;
		url += '&lysate=' + frm.lysates[index].lysate.name;
		Dialogs.openPopup(url, 'EditDnaRnaDetails'+index, 750, 500);	
	}
	
	exreg.downloadSampleIdFile = function()
	{
		var frm = document.forms['reggie'];

		if (!qiacubePositionsAreValid)
		{
			Forms.showNotification('downloadSampleIdFile', 'Qiacube positions are not valid. Please fix before downloading the file.');
			return;
		}
		
		var lysateNames = [];
		var frm = document.forms['reggie'];
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=DownloadNanoDropSampleIdFile';
		url += '&qiacubeDate='+encodeURIComponent(frm.qiacubeDate.value);
		url += '&qiacubeRunNo='+encodeURIComponent(frm.qiacubeRunNo.value);
		
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				var lysate = frm.lysates[i].lysate;
				lysateNames[lysate.qiacubePosition-1] = lysate.name;
			}
		}
		url += '&lysateNames=' + encodeURIComponent(lysateNames.join(','));
		window.open(url);
	}

	exreg.nanoDropFileOnChange = function()
	{
		var frm = document.forms['reggie'];
		var file = frm.nanoDropFile.files[0];
		
		var reader = new FileReader();
		reader.onload = exreg.nanoDropFileLoaded;
		reader.readAsText(file);
	}

	exreg.nanoDropFileLoaded = function(event)
	{
		exreg.parseNanoDropFile(event.target.result);
	}


	exreg.parseNanoDropFile = function(data)
	{
		var frm = document.forms['reggie'];
		nanoDropFileIsValid = false;

		try
		{
			// Reset the existing values
			for (var i = 0; i < frm.lysates.length; i++)
			{
				if (frm.lysates[i].selected)
				{
					var lysate = frm.lysates[i].lysate;
					lysate.dna.ndConc = null;
					lysate.dna.nd260by280 = null;
					lysate.dna.nd260by230 = null;
					lysate.rna.ndConc = null;
					lysate.rna.nd260by280 = null;
					lysate.rna.nd260by230 = null;
				}
			}
			
			var lines = data.split(/[\n\r]+/);
			var columns = lines[0].split(/ *\t */);
			
			var sampleIndex = exreg.findColumn(columns, 'Sample ID');
			if (sampleIndex == -1) return;
			
			var wellIndex = exreg.findColumn(columns, 'Well');
			if (wellIndex == -1) return;
			
			var ndConcIndex = exreg.findColumn(columns, 'Conc.');
			if (ndConcIndex == -1) return;
			
			var nd260by280Index = exreg.findColumn(columns, '260/280');
			if (nd260by280Index == -1) return;
			
			var nd260by230Index = exreg.findColumn(columns, '260/230');
			if (nd260by230Index == -1) return;
		
			var numImported = 0;		
			for (var i = 1; i < lines.length; i++)
			{
				if (lines[i].match(/^\s*$/)) continue; // Ignore empty lines
				
				var row = lines[i].split(/ *\t */);
				if (row.length != columns.length) 
				{
					Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; Unexpected number of columns: ' + row.length);
					return;
				}
				
				var sampleId = row[sampleIndex];
				if (!sampleId || sampleId == 'blank') continue; // with the next line
		
				var rnaOrDna = exreg.findRnaOrDna(sampleId, i+1);
				if (!rnaOrDna) return;  // This is an error condition
				
				if (rnaOrDna.well != row[wellIndex])
				{
					Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; Expected to find "' + sampleId + '" in well [' + rnaOrDna.well + '] not ['  + row[wellIndex] + ']');
					return;
				}
				
				var ndConc = exreg.getNumber(row[ndConcIndex]);
				if (isNaN(ndConc))
				{
					Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; No valid NDConc value for "' + sampleId + '": ' + row[ndConcIndex]);
					return;
				}
				
				// If there are multiple entries for the same item, keep the largest NDConc
				var isFirstValue = rnaOrDna.ndConc == null;
				if (isFirstValue || ndConc > rnaOrDna.ndConc)
				{
					var nd260by280 = exreg.getNumber(row[nd260by280Index]);
					if (isNaN(nd260by280))
					{
						Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; No valid 260/280 value for "' + sampleId + '": ' + row[nd260by280Index]);
						return;
					}
					
					var nd260by230 = exreg.getNumber(row[nd260by230Index]);
					if (isNaN(nd260by230))
					{
						Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + (i+1) + '; No valid 260/230 value for "' + sampleId + '": '+row[nd260by230Index]);
						return;
					}
					
					// Store values
					rnaOrDna.ndConc = ndConc;
					rnaOrDna.nd260by280 = nd260by280;
					rnaOrDna.nd260by230 = nd260by230;

					// Only update counter for the first value (not when replacing a lower value)
					if (isFirstValue)
					{
						numImported++;
					}
				}
				
			}
		
			if (numImported != 24)
			{
				Wizard.setInputStatus('nanoDropFile', 'invalid', 'Could not find values for all 24 samples: ' + numImported);
				return;
			}

			Wizard.setInputStatus('nanoDropFile', 'valid');
			nanoDropFileIsValid = true;
		}
		finally
		{
			exreg.updateDetailsTable();
		}
		
	}

	exreg.findColumn = function(columns, col)
	{
		var colIndex = columns.indexOf(col);
		if (colIndex == -1)
		{
			Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line 1; Can\'t find "' + col + '" column');
		}
		return colIndex;
	}
	
	exreg.getNumber = function(sValue)
	{
		return parseFloat(sValue.replace(/,/, '.'));
	}

	exreg.findRnaOrDna = function(sampleId, lineNo)
	{
		var frm = document.forms['reggie'];
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				var lysate = frm.lysates[i].lysate;
				if (lysate.rna.name == sampleId)
				{
					return lysate.rna;
				}
				else if (lysate.dna.name == sampleId)
				{
					return lysate.dna;
				}
			}
		}
		
		Wizard.setInputStatus('nanoDropFile', 'invalid', 'On line ' + lineNo + '; Can\'t find RNA/DNA with name "' + sampleId + '"');
		return null;
	}
	
	
	exreg.generateFakeNanoDropFile = function()
	{
		if (location.protocol == 'https:')
		{
			Forms.showNotification('btnFakeNanoFropFile', 'For security reasons, this function can\'t be used on HTTPS servers.');
			return;
		}
		
		var FILE_HEADER = ['Well', 'Sample ID', 'Conc.', '260/280', '260/230'];
		var data = FILE_HEADER.join('\t')+'\n';
		
		var frm = document.forms['reggie'];
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				var lysate = frm.lysates[i].lysate;
				var line = [];
				// DNA 
				line[0] = lysate.dna.well;
				line[1] = lysate.dna.name;
				line[2] = Numbers.formatNumber(Math.random()*200+5, 2);
				line[3] = Numbers.formatNumber(Math.random()*.5+2, 2);
				line[4] = Numbers.formatNumber(Math.random()+1, 2);
				data += line.join('\t') + '\n';
				// RNA
				line[0] = lysate.rna.well;
				line[1] = lysate.rna.name;
				line[2] = Numbers.formatNumber(Math.random()*200+5, 2);
				line[3] = Numbers.formatNumber(Math.random()*.5+2, 2);
				line[4] = Numbers.formatNumber(Math.random()+1, 2);
				data += line.join('\t') + '\n';
				
			}
		}
		exreg.parseNanoDropFile(data);
	}

	exreg.validateStep3 = function(event)
	{
		var valid = true;
		valid &= nanoDropFileIsValid;
		valid &= qiacubePositionsAreValid;
		
		if (!valid) event.preventDefault();
	}


	exreg.submit = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		submitInfo.lysates = [];
		
		for (var i = 0; i < frm.lysates.length; i++)
		{
			if (frm.lysates[i].selected)
			{
				var lysate = frm.lysates[i].lysate;
				lysate.rna.preNormalize = frm['rnaPreNormalize'+i].checked;
				submitInfo.lysates[submitInfo.lysates.length] = frm.lysates[i].lysate;
			}
		}
		
		// Lysis
		submitInfo.lysisDate = frm.lysisDate.value;
		submitInfo.lysisProtocol = parseInt(frm.lysisProtocol.value, 10);
		
		// Qiacube
		submitInfo.qiacubeDate = frm.qiacubeDate.value;
		submitInfo.qiacubeRunNo = parseInt(frm.qiacubeRunNo.value, 10);
		submitInfo.qiacubeOperator = frm.qiacubeOperator.value;
		submitInfo.dnaProtocol = parseInt(frm.dnaProtocol.value, 10);
		submitInfo.rnaProtocol = parseInt(frm.rnaProtocol.value, 10);
		submitInfo.ftProtocol = parseInt(frm.ftProtocol.value, 10);

		// Reagents
		for (var i = 0; i < exreg.REAGENTS.length; i++)
		{
			var rg = exreg.REAGENTS[i];
			submitInfo[rg] = frm[rg].value;
		}
		
		var url = '../Extraction.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterDnaRnaFt';
		Wizard.showLoadingAnimation('Performing registration...');
		Wizard.asyncJsonRequest(url, exreg.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	exreg.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
		if (response.createdRna && response.createdRna.length > 0)
		{
			createdRna = response.createdRna;
			Doc.show('rnaQcWizard');	
		}
	}

	exreg.goRnaQc = function()
	{
		var url = 'rnaqc_aliquot.jsp?ID='+App.getSessionId();
		for (var i = 0; i < createdRna.length; i++)
		{
			url += '&rna='+createdRna[i];
		}
		location.href = url;
	}

	
	return exreg;
}();

Doc.onLoad(ExtractionReg.initPage);

